/*
    Authenticate user/passwd
    Copyleft by SATOH Fumiyasu <fumiya@cij.co.jp>
    since 1997/05/23
*/

#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <pwd.h>
#ifdef DO_SHADOWPASSWD
#include <shadow.h>
#endif /* DO_SHADOWPASSWD */
#ifdef HAVE_CRYPT_H
#include <crypt.h>
#endif /* HAVE_CRYPT_H */

#include "trace.h"

#include "authpasswd.h"


/* user/passwd is correct ? */
RESULT AuthenticatePasswd(user_in, passwd_in)
char *user_in, *passwd_in;
{
    char	*passwd_crypt, *passwd;
    int		passwd_diff;
#ifdef DO_SHADOWPASSWD
    struct spwd	*shadow_pwd;
#else /* DO_SHADOWPASSWD */
    struct passwd *normal_pwd;
#endif /* DO_SHADOWPASSWD */

    /* get user entry */
#ifdef DO_SHADOWPASSWD
    if (geteuid() != 0) {
	TRACE(1, ("permission denied. (super-user only)\n"));
	return FAILURE;
    }
    shadow_pwd = getspnam(user_in);
    if (shadow_pwd == NULL) {
	TRACE(1, ("unable to get shadow password entry for %s.\n", user_in));
	return FAILURE;
    }
    passwd = shadow_pwd->sp_pwdp;
#else /* DO_SHADOWPASSWD */
    normal_pwd = getpwnam(user_in);
    if (normal_pwd == NULL) {
	TRACE(1, ("unable to get password entry for %s.\n", user_in));
	return FAILURE;
    }
    passwd = normal_pwd->pw_passwd;
#endif /* DO_SHADOWPASSWD */

    /* check passwd */
    passwd_crypt = crypt(passwd_in, passwd);
    passwd_diff = strcmp(passwd, passwd_crypt);
    TRACE(2, ("[%s][%s][%s][%s]\n", user_in, passwd_in, passwd, passwd_crypt));
    TRACE(2, ("[%d]\n", passwd_diff));

    return (passwd_diff == 0) ? SUCCESS : FAILURE;
}

