/*
    trace.h
    Copyleft by SATOH Fumiyasu <fumiya@cij.co.jp>
*/

#ifndef _TRACE_H_INCLUDED
#define _TRACE_H_INCLUDED

#include "define.h"

#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <errno.h>

#ifdef HAVE_VARARGS_H
# include <varargs.h>
#endif /* HAVE_VARARGS_H */

#ifndef HAVE_STRERROR
# define strerror(errno)	(sys_errlist[errno])
  extern char *sys_errlist[];
#endif /* ! HAVE_STRERROR */



#ifdef __STDC__
void trace_Print _((const char *fmt, /* [arg,] */ ...));
#else /* __STDC__ */
void trace_Print _((int va_alist));
#endif /* __STDC__ */


#define TRACE(level, args) \
    do { \
	if (trace_level >= (level)) { \
	    trace_Print("%s:%s:%d **", trace_module, __FILE__, __LINE__); \
	    trace_Print args ; \
	} \
    } while (0)

#define TRACE_STRERROR(level, args) \
    do { \
	if (trace_level >= (level)) { \
	    trace_Print("%s:%s:%d **", trace_module, __FILE__, __LINE__); \
	    if (errno != 0) { \
		trace_Print("%s **", strerror(errno)); \
		errno = 0; \
	    } \
	    trace_Print args ; \
	} \
    } while (0)

#define TRACE_IF(level)	if (trace_level >= (level))

/* printf for TRACE_IF macro */
#define TRACE_PRINTF(args) \
	trace_Print args


#define TRACE_SETFILE(file)	trace_file = (file)
#define TRACE_SETLEVEL(level)	trace_level = (level)
#define TRACE_SETMODULE(module)	trace_module = (module)

#define TRACE_GETFILE()		(trace_file)
#define TRACE_GETLEVEL()	(trace_level)
#define TRACE_GETMODULE()	(trace_module)

/* current status */
extern  int     trace_level;
extern  char	*trace_module;
extern  FILE	*trace_file;


/* kind of level */
enum trace_types {
    TRACE_T_FATAL = 0,		/* fatal	- die! */
    TRACE_T_ERROR,		/* error	- oh no! */
    TRACE_T_TRACE,		/* trace	- chase! */

    TRACE_T_ERROR_FILEIO,	/* file I/O	- fopen, fread, ... */
    TRACE_T_ERROR_SOCKETIO,	/* socket I/O	- read, write, ... */
    TRACE_T_ERROR_SIGNAL,	/* signal	- ???? */
    TRACE_T_ERROR_PROCESS,	/* process	- fork, kill, ... */
    TRACE_T_ERROR_MEMORY,	/* memory	- malloc, realloc, ... */
    TRACE_T_ERROR_SQL,		/* SQL		- ???? */
    TRACE_T_ERROR_MISC,		/* misc error	- etc, etc, ... */

    TRACE_T_MAX			/* maximum level */
};

#endif /* _TRACE_H_INCLUDED */
