/*
    xpasswd - non-interactive passwd command
    Copyleft by SATOH Fumiyasu <fumiya@cij.co.jp>

    version: 1.0.1 beta6 (1997/12/12 - since 1997/05/23)
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pwd.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "trace.h"
#include "authpasswd.h"
#include "chpasswd.h"

#include "xpasswd.h"


int main(int argc, char **argv);
static void ReadString(char *buf, char **pstr, int len);
static void TruncateString(char *str, int len);

int main(int argc, char **argv)
{
    int		i;
    extern int	getopt(int, char * const *, const char *);
    extern char	*optarg;
    extern int	optind;
    int		opt, opt_a=0, opt_d=0, opt_v=0;
    struct passwd *result;
    char	user_buf[MAXIN], pass_buf[MAXIN], newpass_buf[MAXIN];
    char	*user_in, *pass_in, *newpass_in;
    char	*dir;

    TRACE_SETLEVEL(0);
    TRACE_SETMODULE("xpasswd");

    /* paranoia ? */
    argc = (argc < MAXARGC) ? argc : MAXARGC;
    for (i=0; i<argc; i++) {
	TruncateString(argv[i], MAXARGVLEN);
    }

    /* analyze opitons */
    while ((opt = getopt(argc, argv, "advh")) != EOF) {
	switch (opt) {
	    case 'a':
		opt_a = 1;
		break;
	    case 'd':
		opt_d = 1;
		break;
	    case 'v':
		opt_v++;
		TRACE_SETLEVEL(opt_v);
		break;
	    case 'h':
		puts("xpasswd - non-interactive passwd command");
		puts("Copyleft by SATOH Fumiyasu <fumiya@cij.co.jp>");
		puts("usage:");
		puts("\txpasswd [-a] [-d] [-v[v]]");
		puts("options:");
		puts("\t-a\tauthentication only");
		puts("\t-d\tcheck existing home directory");
		puts("\t-v\tverbose output");
		puts("\t-h\tshow this message");
		puts("return:");
		puts("\t0\tsuccess !");
		puts("\t1\tusername, password or new password is not there");
		puts("\t2\tusername or password is incorrect");
		puts("\t3\tuser's home directory does not exist");
		puts("\t4\tnew password is not suitable string");
		puts("\t5\tpasswd changing failed (unknown error)");
		exit(-1);
	    default:
		puts("usage:xpasswd [-a] [-d] [-v[v]]");
		exit(-1);
	}
    }

    /* get user and passwd */
    ReadString(user_buf, &user_in, MAXIN);
    if ((user_in == NULL) || (strlen(user_in) == 0)) {
	TRACE(1, ("username is not there.\n"));
	exit(1);
    }
    ReadString(pass_buf, &pass_in, MAXIN);
    if (pass_in == NULL) {
	TRACE(1, ("password is not there.\n"));
	exit(1);
    }

    /* get user entry */
    result = getpwnam(user_in);
    if (result == NULL) {
	TRACE(1, ("unalbe to get password entry for %s.\n", user_in));
	exit(2);
    }
    dir = result->pw_dir;

    /* check home directory */
    if (opt_d) {
	struct stat dir_stat;

	if ((stat(dir, &dir_stat) == -1) || !(dir_stat.st_mode & S_IFDIR)) {
	    TRACE(1, ("%s's home directory does not exist.\n", user_in));
	    exit(3);
	}
    }

    /* check user/passwd */
    if (AuthenticatePasswd(user_in, pass_in) != SUCCESS) {
	TRACE(1, ("username or password is incorrect.\n"));
	exit(2);
    }

    /* authentication only ? */
    if (opt_a) {
	exit(0);
    }


    /* get new passwd */
    ReadString(newpass_buf, &newpass_in, MAXIN);
    if (newpass_in == NULL) {
	TRACE(1, ("new password is not there.\n"));
	exit(1);
	exit(1);
    }

    if (IsSuitablePasswd(newpass_in, user_in, pass_in) != TRUE) {
	TRACE(1, ("new password is not suitable\n"));
	exit(4);
    }
    if (ChangePasswd(user_in, pass_in, newpass_in) != SUCCESS) {
	TRACE(1, ("passwd changing failed. (unkown error)\n"));
	exit(5);
    }

#if 0
    /* If you use NIS or NIS+, do NOT use this section... */
    /* Why? Because, NIS(+) server is NOT updated AT ONCE. */

    /* check user/new-passwd */
    if (AuthenticatePasswd(user_in, newpass_in) != SUCCESS) {
	TRACE(1, ("Passwd changing failed.\n"));
	exit(5);
    }
#endif /* 0 */

    exit(0);
}

/* truncate string */
static void TruncateString(str, len)
char *str;
int len;
{
    if (strlen(str) > len) {
	str[len] = '\0';
    }
}

/* read string from stdin */
static void ReadString(buf, pstr, len)
char *buf;
char **pstr;
int  len;
{
    char *cr;

    buf[0] = '\0';
    fgets(buf, len, stdin);
    if (cr = strchr(buf, '\n')) {
	*cr = '\0';
    }
    if (*pstr = strchr(buf, '=')) {
	(*pstr)++;
    }
}
